# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rservice.proto\x12\x08\x61gbox.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\r\n\x0bPingRequest\"/\n\x0cPingResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x0e\n\x06\x64\x61\x65mon\x18\x02 \x01(\t\"&\n\x08KeyValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"=\n\tMountSpec\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0e\n\x06target\x18\x02 \x01(\t\x12\x10\n\x08writable\x18\x03 \x01(\x08\"D\n\x08\x43opySpec\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0e\n\x06target\x18\x02 \x01(\t\x12\x18\n\x10\x65xclude_patterns\x18\x03 \x03(\t\"\x83\x01\n\x11HealthcheckConfig\x12\x0c\n\x04test\x18\x01 \x03(\t\x12\x10\n\x08interval\x18\x02 \x01(\t\x12\x0f\n\x07timeout\x18\x03 \x01(\t\x12\x0f\n\x07retries\x18\x04 \x01(\r\x12\x14\n\x0cstart_period\x18\x05 \x01(\t\x12\x16\n\x0estart_interval\x18\x06 \x01(\t\"\xa4\x01\n\x0bServiceSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05image\x18\x02 \x01(\t\x12\'\n\x0b\x65nvironment\x18\x03 \x03(\x0b\x32\x12.agbox.v1.KeyValue\x12\x30\n\x0bhealthcheck\x18\x04 \x01(\x0b\x32\x1b.agbox.v1.HealthcheckConfig\x12\x1d\n\x15post_start_on_primary\x18\x05 \x03(\t\"\xc4\x02\n\nCreateSpec\x12\r\n\x05image\x18\x01 \x01(\t\x12#\n\x06mounts\x18\x02 \x03(\x0b\x32\x13.agbox.v1.MountSpec\x12\"\n\x06\x63opies\x18\x03 \x03(\x0b\x32\x12.agbox.v1.CopySpec\x12\x19\n\x11\x62uiltin_resources\x18\x04 \x03(\t\x12\x30\n\x11required_services\x18\x05 \x03(\x0b\x32\x15.agbox.v1.ServiceSpec\x12\x30\n\x11optional_services\x18\x06 \x03(\x0b\x32\x15.agbox.v1.ServiceSpec\x12\x30\n\x06labels\x18\x07 \x03(\x0b\x32 .agbox.v1.CreateSpec.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xaf\x02\n\rSandboxHandle\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12%\n\x05state\x18\x02 \x01(\x0e\x32\x16.agbox.v1.SandboxState\x12\x1b\n\x13last_event_sequence\x18\x03 \x01(\x04\x12\x30\n\x11required_services\x18\x04 \x03(\x0b\x32\x15.agbox.v1.ServiceSpec\x12\x30\n\x11optional_services\x18\x05 \x03(\x0b\x32\x15.agbox.v1.ServiceSpec\x12\x33\n\x06labels\x18\x06 \x03(\x0b\x32#.agbox.v1.SandboxHandle.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9e\x03\n\x0cSandboxEvent\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x10\n\x08sequence\x18\x02 \x01(\x04\x12\x12\n\nsandbox_id\x18\x03 \x01(\t\x12\'\n\nevent_type\x18\x04 \x01(\x0e\x32\x13.agbox.v1.EventType\x12/\n\x0boccurred_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06replay\x18\x06 \x01(\x08\x12\x10\n\x08snapshot\x18\x07 \x01(\x08\x12\r\n\x05phase\x18\x08 \x01(\t\x12\x12\n\nerror_code\x18\t \x01(\t\x12\x15\n\rerror_message\x18\n \x01(\t\x12\x0e\n\x06reason\x18\x0b \x01(\t\x12\x0f\n\x07\x65xec_id\x18\x0c \x01(\t\x12\x11\n\texit_code\x18\r \x01(\x05\x12-\n\rsandbox_state\x18\x0e \x01(\x0e\x32\x16.agbox.v1.SandboxState\x12\'\n\nexec_state\x18\x0f \x01(\x0e\x32\x13.agbox.v1.ExecState\x12\x14\n\x0cservice_name\x18\x10 \x01(\t\"\xdd\x01\n\nExecStatus\x12\x0f\n\x07\x65xec_id\x18\x01 \x01(\t\x12\x12\n\nsandbox_id\x18\x02 \x01(\t\x12\"\n\x05state\x18\x03 \x01(\x0e\x32\x13.agbox.v1.ExecState\x12\x0f\n\x07\x63ommand\x18\x04 \x03(\t\x12\x0b\n\x03\x63wd\x18\x05 \x01(\t\x12)\n\renv_overrides\x18\x06 \x03(\x0b\x32\x12.agbox.v1.KeyValue\x12\x11\n\texit_code\x18\x07 \x01(\x05\x12\r\n\x05\x65rror\x18\x08 \x01(\t\x12\x1b\n\x13last_event_sequence\x18\t \x01(\x04\"j\n\x14\x43reateSandboxRequest\x12)\n\x0b\x63reate_spec\x18\x01 \x01(\x0b\x32\x14.agbox.v1.CreateSpec\x12\x12\n\nsandbox_id\x18\x02 \x01(\t\x12\x13\n\x0b\x63onfig_yaml\x18\x03 \x01(\x0c\"Z\n\x15\x43reateSandboxResponse\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12-\n\rinitial_state\x18\x02 \x01(\x0e\x32\x16.agbox.v1.SandboxState\"\'\n\x11GetSandboxRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\">\n\x12GetSandboxResponse\x12(\n\x07sandbox\x18\x01 \x01(\x0b\x32\x17.agbox.v1.SandboxHandle\"\xb0\x01\n\x14ListSandboxesRequest\x12\x17\n\x0finclude_deleted\x18\x01 \x01(\x08\x12I\n\x0elabel_selector\x18\x02 \x03(\x0b\x32\x31.agbox.v1.ListSandboxesRequest.LabelSelectorEntry\x1a\x34\n\x12LabelSelectorEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"C\n\x15ListSandboxesResponse\x12*\n\tsandboxes\x18\x01 \x03(\x0b\x32\x17.agbox.v1.SandboxHandle\"*\n\x14ResumeSandboxRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\"(\n\x12StopSandboxRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\"*\n\x14\x44\x65leteSandboxRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\"\x9b\x01\n\x16\x44\x65leteSandboxesRequest\x12K\n\x0elabel_selector\x18\x01 \x03(\x0b\x32\x33.agbox.v1.DeleteSandboxesRequest.LabelSelectorEntry\x1a\x34\n\x12LabelSelectorEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"M\n\x17\x44\x65leteSandboxesResponse\x12\x1b\n\x13\x64\x65leted_sandbox_ids\x18\x01 \x03(\t\x12\x15\n\rdeleted_count\x18\x02 \x01(\r\"$\n\x10\x41\x63\x63\x65ptedResponse\x12\x10\n\x08\x61\x63\x63\x65pted\x18\x01 \x01(\x08\"l\n\x1dSubscribeSandboxEventsRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\x15\n\rfrom_sequence\x18\x02 \x01(\x04\x12 \n\x18include_current_snapshot\x18\x03 \x01(\x08\"\x81\x01\n\x11\x43reateExecRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\x0f\n\x07\x63ommand\x18\x02 \x03(\t\x12\x0b\n\x03\x63wd\x18\x03 \x01(\t\x12)\n\renv_overrides\x18\x04 \x03(\x0b\x32\x12.agbox.v1.KeyValue\x12\x0f\n\x07\x65xec_id\x18\x05 \x01(\t\"W\n\x12\x43reateExecResponse\x12\x0f\n\x07\x65xec_id\x18\x01 \x01(\t\x12\x17\n\x0fstdout_log_path\x18\x02 \x01(\t\x12\x17\n\x0fstderr_log_path\x18\x03 \x01(\t\"$\n\x11\x43\x61ncelExecRequest\x12\x0f\n\x07\x65xec_id\x18\x01 \x01(\t\"!\n\x0eGetExecRequest\x12\x0f\n\x07\x65xec_id\x18\x01 \x01(\t\"5\n\x0fGetExecResponse\x12\"\n\x04\x65xec\x18\x01 \x01(\x0b\x32\x14.agbox.v1.ExecStatus\",\n\x16ListActiveExecsRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\">\n\x17ListActiveExecsResponse\x12#\n\x05\x65xecs\x18\x01 \x03(\x0b\x32\x14.agbox.v1.ExecStatus*\xcd\x01\n\x0cSandboxState\x12\x1d\n\x19SANDBOX_STATE_UNSPECIFIED\x10\x00\x12\x19\n\x15SANDBOX_STATE_PENDING\x10\x01\x12\x17\n\x13SANDBOX_STATE_READY\x10\x02\x12\x18\n\x14SANDBOX_STATE_FAILED\x10\x03\x12\x19\n\x15SANDBOX_STATE_STOPPED\x10\x04\x12\x1a\n\x16SANDBOX_STATE_DELETING\x10\x05\x12\x19\n\x15SANDBOX_STATE_DELETED\x10\x06*\xe0\x02\n\tEventType\x12\x1a\n\x16\x45VENT_TYPE_UNSPECIFIED\x10\x00\x12\x14\n\x10SANDBOX_ACCEPTED\x10\x01\x12\x15\n\x11SANDBOX_PREPARING\x10\x02\x12\x11\n\rSANDBOX_READY\x10\x03\x12\x12\n\x0eSANDBOX_FAILED\x10\x04\x12\x1a\n\x16SANDBOX_STOP_REQUESTED\x10\x05\x12\x13\n\x0fSANDBOX_STOPPED\x10\x06\x12\x1c\n\x18SANDBOX_DELETE_REQUESTED\x10\x07\x12\x13\n\x0fSANDBOX_DELETED\x10\x08\x12\x10\n\x0c\x45XEC_STARTED\x10\t\x12\x11\n\rEXEC_FINISHED\x10\n\x12\x0f\n\x0b\x45XEC_FAILED\x10\x0b\x12\x12\n\x0e\x45XEC_CANCELLED\x10\x0c\x12\x19\n\x15SANDBOX_SERVICE_READY\x10\r\x12\x1a\n\x16SANDBOX_SERVICE_FAILED\x10\x0e*\x89\x01\n\tExecState\x12\x1a\n\x16\x45XEC_STATE_UNSPECIFIED\x10\x00\x12\x16\n\x12\x45XEC_STATE_RUNNING\x10\x01\x12\x17\n\x13\x45XEC_STATE_FINISHED\x10\x02\x12\x15\n\x11\x45XEC_STATE_FAILED\x10\x03\x12\x18\n\x14\x45XEC_STATE_CANCELLED\x10\x04\x32\xf4\x07\n\x0eSandboxService\x12\x35\n\x04Ping\x12\x15.agbox.v1.PingRequest\x1a\x16.agbox.v1.PingResponse\x12P\n\rCreateSandbox\x12\x1e.agbox.v1.CreateSandboxRequest\x1a\x1f.agbox.v1.CreateSandboxResponse\x12G\n\nGetSandbox\x12\x1b.agbox.v1.GetSandboxRequest\x1a\x1c.agbox.v1.GetSandboxResponse\x12P\n\rListSandboxes\x12\x1e.agbox.v1.ListSandboxesRequest\x1a\x1f.agbox.v1.ListSandboxesResponse\x12K\n\rResumeSandbox\x12\x1e.agbox.v1.ResumeSandboxRequest\x1a\x1a.agbox.v1.AcceptedResponse\x12G\n\x0bStopSandbox\x12\x1c.agbox.v1.StopSandboxRequest\x1a\x1a.agbox.v1.AcceptedResponse\x12K\n\rDeleteSandbox\x12\x1e.agbox.v1.DeleteSandboxRequest\x1a\x1a.agbox.v1.AcceptedResponse\x12V\n\x0f\x44\x65leteSandboxes\x12 .agbox.v1.DeleteSandboxesRequest\x1a!.agbox.v1.DeleteSandboxesResponse\x12[\n\x16SubscribeSandboxEvents\x12\'.agbox.v1.SubscribeSandboxEventsRequest\x1a\x16.agbox.v1.SandboxEvent0\x01\x12G\n\nCreateExec\x12\x1b.agbox.v1.CreateExecRequest\x1a\x1c.agbox.v1.CreateExecResponse\x12\x45\n\nCancelExec\x12\x1b.agbox.v1.CancelExecRequest\x1a\x1a.agbox.v1.AcceptedResponse\x12>\n\x07GetExec\x12\x18.agbox.v1.GetExecRequest\x1a\x19.agbox.v1.GetExecResponse\x12V\n\x0fListActiveExecs\x12 .agbox.v1.ListActiveExecsRequest\x1a!.agbox.v1.ListActiveExecsResponseBDZBgithub.com/1996fanrui/agents-sandbox/api/generated/agboxv1;agboxv1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZBgithub.com/1996fanrui/agents-sandbox/api/generated/agboxv1;agboxv1'
  _globals['_CREATESPEC_LABELSENTRY']._loaded_options = None
  _globals['_CREATESPEC_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_SANDBOXHANDLE_LABELSENTRY']._loaded_options = None
  _globals['_SANDBOXHANDLE_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_LISTSANDBOXESREQUEST_LABELSELECTORENTRY']._loaded_options = None
  _globals['_LISTSANDBOXESREQUEST_LABELSELECTORENTRY']._serialized_options = b'8\001'
  _globals['_DELETESANDBOXESREQUEST_LABELSELECTORENTRY']._loaded_options = None
  _globals['_DELETESANDBOXESREQUEST_LABELSELECTORENTRY']._serialized_options = b'8\001'
  _globals['_SANDBOXSTATE']._serialized_start=3400
  _globals['_SANDBOXSTATE']._serialized_end=3605
  _globals['_EVENTTYPE']._serialized_start=3608
  _globals['_EVENTTYPE']._serialized_end=3960
  _globals['_EXECSTATE']._serialized_start=3963
  _globals['_EXECSTATE']._serialized_end=4100
  _globals['_PINGREQUEST']._serialized_start=60
  _globals['_PINGREQUEST']._serialized_end=73
  _globals['_PINGRESPONSE']._serialized_start=75
  _globals['_PINGRESPONSE']._serialized_end=122
  _globals['_KEYVALUE']._serialized_start=124
  _globals['_KEYVALUE']._serialized_end=162
  _globals['_MOUNTSPEC']._serialized_start=164
  _globals['_MOUNTSPEC']._serialized_end=225
  _globals['_COPYSPEC']._serialized_start=227
  _globals['_COPYSPEC']._serialized_end=295
  _globals['_HEALTHCHECKCONFIG']._serialized_start=298
  _globals['_HEALTHCHECKCONFIG']._serialized_end=429
  _globals['_SERVICESPEC']._serialized_start=432
  _globals['_SERVICESPEC']._serialized_end=596
  _globals['_CREATESPEC']._serialized_start=599
  _globals['_CREATESPEC']._serialized_end=923
  _globals['_CREATESPEC_LABELSENTRY']._serialized_start=878
  _globals['_CREATESPEC_LABELSENTRY']._serialized_end=923
  _globals['_SANDBOXHANDLE']._serialized_start=926
  _globals['_SANDBOXHANDLE']._serialized_end=1229
  _globals['_SANDBOXHANDLE_LABELSENTRY']._serialized_start=878
  _globals['_SANDBOXHANDLE_LABELSENTRY']._serialized_end=923
  _globals['_SANDBOXEVENT']._serialized_start=1232
  _globals['_SANDBOXEVENT']._serialized_end=1646
  _globals['_EXECSTATUS']._serialized_start=1649
  _globals['_EXECSTATUS']._serialized_end=1870
  _globals['_CREATESANDBOXREQUEST']._serialized_start=1872
  _globals['_CREATESANDBOXREQUEST']._serialized_end=1978
  _globals['_CREATESANDBOXRESPONSE']._serialized_start=1980
  _globals['_CREATESANDBOXRESPONSE']._serialized_end=2070
  _globals['_GETSANDBOXREQUEST']._serialized_start=2072
  _globals['_GETSANDBOXREQUEST']._serialized_end=2111
  _globals['_GETSANDBOXRESPONSE']._serialized_start=2113
  _globals['_GETSANDBOXRESPONSE']._serialized_end=2175
  _globals['_LISTSANDBOXESREQUEST']._serialized_start=2178
  _globals['_LISTSANDBOXESREQUEST']._serialized_end=2354
  _globals['_LISTSANDBOXESREQUEST_LABELSELECTORENTRY']._serialized_start=2302
  _globals['_LISTSANDBOXESREQUEST_LABELSELECTORENTRY']._serialized_end=2354
  _globals['_LISTSANDBOXESRESPONSE']._serialized_start=2356
  _globals['_LISTSANDBOXESRESPONSE']._serialized_end=2423
  _globals['_RESUMESANDBOXREQUEST']._serialized_start=2425
  _globals['_RESUMESANDBOXREQUEST']._serialized_end=2467
  _globals['_STOPSANDBOXREQUEST']._serialized_start=2469
  _globals['_STOPSANDBOXREQUEST']._serialized_end=2509
  _globals['_DELETESANDBOXREQUEST']._serialized_start=2511
  _globals['_DELETESANDBOXREQUEST']._serialized_end=2553
  _globals['_DELETESANDBOXESREQUEST']._serialized_start=2556
  _globals['_DELETESANDBOXESREQUEST']._serialized_end=2711
  _globals['_DELETESANDBOXESREQUEST_LABELSELECTORENTRY']._serialized_start=2302
  _globals['_DELETESANDBOXESREQUEST_LABELSELECTORENTRY']._serialized_end=2354
  _globals['_DELETESANDBOXESRESPONSE']._serialized_start=2713
  _globals['_DELETESANDBOXESRESPONSE']._serialized_end=2790
  _globals['_ACCEPTEDRESPONSE']._serialized_start=2792
  _globals['_ACCEPTEDRESPONSE']._serialized_end=2828
  _globals['_SUBSCRIBESANDBOXEVENTSREQUEST']._serialized_start=2830
  _globals['_SUBSCRIBESANDBOXEVENTSREQUEST']._serialized_end=2938
  _globals['_CREATEEXECREQUEST']._serialized_start=2941
  _globals['_CREATEEXECREQUEST']._serialized_end=3070
  _globals['_CREATEEXECRESPONSE']._serialized_start=3072
  _globals['_CREATEEXECRESPONSE']._serialized_end=3159
  _globals['_CANCELEXECREQUEST']._serialized_start=3161
  _globals['_CANCELEXECREQUEST']._serialized_end=3197
  _globals['_GETEXECREQUEST']._serialized_start=3199
  _globals['_GETEXECREQUEST']._serialized_end=3232
  _globals['_GETEXECRESPONSE']._serialized_start=3234
  _globals['_GETEXECRESPONSE']._serialized_end=3287
  _globals['_LISTACTIVEEXECSREQUEST']._serialized_start=3289
  _globals['_LISTACTIVEEXECSREQUEST']._serialized_end=3333
  _globals['_LISTACTIVEEXECSRESPONSE']._serialized_start=3335
  _globals['_LISTACTIVEEXECSRESPONSE']._serialized_end=3397
  _globals['_SANDBOXSERVICE']._serialized_start=4103
  _globals['_SANDBOXSERVICE']._serialized_end=5115
# @@protoc_insertion_point(module_scope)
